<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 10/11/2003 *
 * ===============                              Last change: 10/11/2003 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : admins_functions.php                             *
 * -------------------------------------------------------------------- *
 * Short description : Functions for the admins extension               *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer die admins-Erweiterung           *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

// Caching class
class mxchange_cache {
	// Define variables
	var $ret = "init";
	var $cache_path = "";
	var $cache_inc = "";
	var $cache_pointer = false;
	var $cache_data = "";
	var $cache_version = "";
	var $cache_file = "";

	// Constructor
	function mxchange_cache($interval, $path, $tested) {
		// Failed is the default
		$this->ret = "failed";

		// Remeber path
		$this->cache_path = $path;

		// Check if path exists
		if ((is_dir($path)) && (!$tested)) {
			// Check if we can create a file inside the path
			touch($path."dummy.tmp", 'w');
			if (FILE_READABLE($path."dummy.tmp")) {
				// Yes, we can do. So let's remove it
				unlink($path."dummy.tmp");

				// Is there a .htaccess file?
				if (FILE_READABLE($path.".htaccess")) {
					// Update database that we have tested it
					UPDATE_CONFIG("cache_tested", 1);

					// All done!
					$this->ret = "done";
				} else {
					// Stop! Set a .htaccess file first
					$this->ret = "htaccess";
				}
			}
		} elseif ($tested) {
			// System already tested
			$this->ret = "done";
		}
	}

	// Checks validity of cache file and if content is given
	function cache_file ($file, $forceContent = false) {
		// Remember cache file
		$this->cache_file = $file;

		// Construct FQFN (full qualified file name)
		$inc = $this->cache_path.$file.".cache";

		// Rember it + filename in class
		$this->cache_inc = $inc;

		// Check if file exists and if version matches
		$status = (FILE_READABLE($inc) && (is_writeable($inc)) && ($this->ext_version_matches("cache")));

		// Return status
		return $status;
	}

	function cache_init($array) {
		// This will destory an existing cache file!
		if ($this->ret == "done") {
			// Create file
			if (FILE_READABLE($this->cache_inc)) chmod($this->cache_inc, 0666);
			$this->cache_pointer = fopen($this->cache_inc, 'w') or mxchange_die("Cannot write to cache ".$this->cache_inc." !");

			// Begin of cache file
			fwrite($this->cache_pointer, "<?php\n\$ARRAY = \"".$array."\";\n\n");

			// Add default depency
			$this->store_extension_version("cache");
		} else {
			// Cannot create file
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function add_row ($data) {
		global $cacheArray;

		if (is_resource($this->cache_pointer)) {
			// Write every array element to cache file
			foreach ($data as $k => $v) {
				// Write global cache array for immediate access
				if ((substr($k, 0, 4) == "ext_") && (isset($data['ext_name'])) && (isset($data['ext_id']))) {
					if ($k != "ext_name") {
						$cacheArray['extensions'][$k][$data['ext_name']] = $v;
					} else {
						$cacheArray['extensions'][$k][$data['ext_id']] = $v;
					}
					if (($k == "ext_keep") && ($v == "Y")) {
						$cacheArray['active_extensions'][$data['ext_name']] = $v;
					} // END - if
				} elseif (is_array($v)) {
					// Serialize and BASE64-encode the array
					$v = base64_encode(serialize($v));
				}

				// Write cache line to file
				fwrite($this->cache_pointer, $this->add_raw_row($k, $v));
			}
		} else {
			// Cannot create file
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_close() {
		// Quit function when no pointer is set
		if (empty($this->cache_pointer)) return;
		if (is_resource($this->cache_pointer)) {
			// Write footer
			fwrite($this->cache_pointer, "?>\n");

			// Close file add destroy handler
			fclose($this->cache_pointer);

			// Set rights
			if (FILE_READABLE($this->cache_inc)) chmod($this->cache_inc, 0666);

			// Remove pointer
			unset($this->cache_pointer);
		} else {
			// Cannot create file
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_load() {
		// Is the cache already loaded?
		if (isset($this->cache_data[$this->cache_file])) {
			// Return it's content!
			return $this->cache_data[$this->cache_file];
		} // END - if

		// Is the cache file there?
		if (FILE_READABLE($this->cache_inc)) {
			// Prepare temporary array
			$data = array();
			$cache_version = null;

			// Load cache file
			require_once($this->cache_inc);

			// Is there an array?
			if (is_array($data)) {
				// Cache data
				$this->cache_data[$this->cache_file] = $data;

				// Cache version found?
				if ((is_array($cache_version)) && (count($cache_version) > 0)) {
					// Remember it as well...
					$this->cache_version[$this->cache_file] = $cache_version;
				} else {
					// Invalid cache so destroy it
					$this->cache_destroy();

					// Clear cached data
					$this->cache_data[$this->cache_file] = array();
				}

				// Return cache
				return $this->cache_data[$this->cache_file];
			} else {
				// Cache problem detected!
				$this->cache_destroy();
			}
		} else {
			// Cache file not found or not readable
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_CANNOT_LOAD_1.$this->cache_inc.CACHE_CANNOT_LOAD_2);
		}
	}

	// Destroy an existing cache file
	function cache_destroy() {
		// Is the cache file there?
		if (FILE_READABLE($this->cache_inc)) {
			// Close cache
			$this->cache_close();

			// Remove cache file from system
			unlink($this->cache_inc);

			// Is the file there?
			if (!FILE_READABLE($this->cache_inc)) {
				// The cache does no longer exist so kill the content
				unset($this->cache_data[$this->cache_file]);
			} else {
				// Not removed!
				ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_CANNOT_UNLINK_1.$this->cache_inc.CACHE_CANNOT_UNLINK_2);
			}
		} else {
			// Does not exist!
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_remove($search, $data, $array) {
		global $ARRAY;
		if ((FILE_READABLE($this->cache_inc)) && (is_writeable($this->cache_inc))) {
			// Load cache into dummy array
			$dummy = $this->cache_load();

			// Search for key in array
			$key = array_search($data, $dummy[$search]);
			if (!empty($key)) {
				// Key (hopefully) found?
				foreach ($array as $a) {
					// So we can remove all elements as requested
					unset($dummy[$a][$key]);
				}

				// Flush array to cache file
				$this->cache_init($ARRAY);

				// Write array out
				$this->cache_write_array($dummy);

				// Close cache file
				$this->cache_close();
			}
		} else {
			// Cannot write to cache!
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_write_array ($array) {
		if (is_resource($this->cache_pointer)) {
			foreach ($array as $k => $v) {
				if (is_array($v)) {
					// Multi line(s) found
					$LINE = "";
					foreach($v as $k2 => $v2) {
						// Put every array element in a row...
						$LINE .= $this->add_raw_row($k, $v2);
					}
				} else {
					// Single line found
					$LINE = $this->add_raw_row($k, $v);
				}

				// Write line(s)
				fwrite($this->cache_pointer, $LINE);
			} // END - foreach
		} else {
			// Cannot write array!
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_replace($search, $replace, $search_key, $array) {
		global $ARRAY;
		if ((FILE_READABLE($this->cache_inc)) && (is_writeable($this->cache_inc))) {
			// Load cache into dummy array
			$dummy = $this->cache_load();

			// Check if $dummy is valid (prevents some errors)
			if ((is_array($dummy)) && (isset($dummy[$search])) && (is_array($dummy[$search]))) {
				// Search for key in array
				$key_found = array_key_exists($search_key, $dummy[$search]);
				if ($key_found == true) {
					$key = $search_key;
					// Key (hopefully) found?
					foreach ($dummy as $a => $v) {
						// So we can update all entries
						if ($a == $search) {
							// Update now...
							$dummy[$a][$search_key] = $replace;
						}
					}

					// Flush array to cache file
					$this->cache_init($ARRAY);

					// Write array out
					$this->cache_write_array($dummy);

					// Close cache file
					$this->cache_close();
				}
			}
		} else {
			// Cannot write to cache!
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function store_extension_version ($ext_name) {
		// Valid cache pointer?
		if (is_resource($this->cache_pointer)) {
			// Get extension version
			$ext_ver = GET_EXT_VERSION($ext_name);

			// Write cache line to file
			fwrite($this->cache_pointer, "\$cache_version['".$ext_name."'] = \"".$ext_ver."\";\n");
		} else {
			// Cannot create file
			ADD_FATAL(__FILE__."(<font color=\"#0000aa\">".__LINE__."</font>): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function ext_version_matches ($ext_name) {
		// Load cache (dummy)
		$this->cache_load();

		// Get extension version
		$ext_ver = GET_EXT_VERSION($ext_name);

		// Compare both
		return ((isset($this->cache_version[$this->cache_file][$ext_name])) && ($this->cache_version[$this->cache_file][$ext_name] == $ext_ver));
	}

	function add_raw_row ($key, $value) {
		// Init line
		$line = "";

		// String or non-string? ;-)
		if ((is_string($value)) || (is_null($value))) {
			// String...
			$line = "\$data['".$key."'][] = \"".$value."\";\n";
		} else {
			// Non-string
			$line = "\$data['".$key."'][] = ".$value.";\n";
		}

		// Return line
		return $line;
	}

	function getStatus () {
		return $this->ret;
	}
}
//
?>
